%% Function to account for behaviour of server in response to local pressure conditions.  Function is called in PFBuoyant16_thesis.

function [EServAdd, VServAdd] = ServerBehaviour_thesis(ServPerRack, Fan25, Fan50, Fan75, Fan100, Rack, i, j, VServ, SFSOption, P_required, dptemp, FansPServer, VServ_0, ServerFanPowerMax, ServerPts, ServerFanPower, c_p, C_D, EServ_0)

VServChange=1;
VServPrev=VServ(i,j);
counter=0;
    
while VServChange>0.01 && counter<20
    counter=counter+1;
    if SFSOption==1 % Server fan speed is assumed to regulate to give a flow rate through the server which ensures dT across the server=12.5K
        if P_required(Rack)-dptemp>=0 % If server fans are required
            Q_F=zeros(4,1); % This to be populated with the flow rates achieved by the fan at this pressure, at 25, 50, 75 and 100% of server fan power
            for m=1:length(Fan25)
                if Fan25(m,2)>P_required(Rack)-dptemp % Server fans need to generate a pressure equal to P_required minus local pressure drop
                    Q_F(1)=Fan25(m-1,1)+(Fan25(m,1)-Fan25(m-1,1))*(P_required(Rack)-dptemp-Fan25(m-1,2))/(Fan25(m,2)-Fan25(m-1,2));
                    break
                end
            end
            for m=1:length(Fan50)
                if Fan50(m,2)>P_required(Rack)-dptemp
                    Q_F(2)=Fan50(m-1,1)+(Fan50(m,1)-Fan50(m-1,1))*(P_required(Rack)-dptemp-Fan50(m-1,2))/(Fan50(m,2)-Fan50(m-1,2));
                    break
                end
            end
            for m=1:length(Fan75)
                if Fan75(m,2)>P_required(Rack)-dptemp
                    Q_F(3)=Fan75(m-1,1)+(Fan75(m,1)-Fan75(m-1,1))*(P_required(Rack)-dptemp-Fan75(m-1,2))/(Fan75(m,2)-Fan75(m-1,2));
                    break
                end
            end
            for m=1:length(Fan100)
                if Fan100(m,2)>P_required(Rack)-dptemp
                    Q_F(4)=Fan100(m-1,1)+(Fan100(m,1)-Fan100(m-1,1))*(P_required(Rack)-dptemp-Fan100(m-1,2))/(Fan100(m,2)-Fan100(m-1,2));
                    break
                end
            end
            for m=1:length(Q_F)
                if FansPServer*(ServPerRack/ServerPts(Rack))*Q_F(m)>VServ(i,j) % if flow achieved at this (25%, 50%, 75% or 100%) server fan power consumption is sufficient
                    if m==1
                        ServerFanPowerTemp=(ServPerRack*ServerFanPowerMax*FansPServer/ServerPts(Rack))*(((ServerPts(Rack)*VServ(i,j)/(FansPServer*ServPerRack))/(Q_F(m)))/4); % server fan power in Watts (divided by 4 because i=1 corresponds to 25% power consumption)
                    else
                        ServerFanPowerTemp=(ServPerRack*ServerFanPowerMax*FansPServer/ServerPts(Rack))*(m-1+(((ServerPts(Rack)*VServ(i,j)/(FansPServer*ServPerRack))-Q_F(m-1))/(Q_F(m)-Q_F(m-1)))/4); % server fan power in Watts (divided by 4 because i=1 corresponds to 25% power consumption)
                    end
                    break
                end

            end
            if FansPServer*(ServPerRack/ServerPts(Rack))*Q_F(m)<=VServ(i,j) % If max fan power doesn't generate enough flow, just set to max fan power
                ServerFanPowerTemp=ServPerRack*ServerFanPowerMax*FansPServer/ServerPts(Rack);
            end
            EServAdd=ServerFanPowerTemp-ServerFanPower(Rack)/ServerPts(Rack); % Difference between server fan power consumption corresponding to this node with current dp, and with dp=0
            VServAdd=(ServerFanPowerTemp-ServerFanPower(Rack)/ServerPts(Rack))/(c_p*12.5); % Flow rate required to remove extra heat
        else % If local pressure exceeds required pressure
            EServAdd=-ServerFanPower(Rack)/ServerPts(Rack); % server fans consume no power
            if P_required(Rack)+dptemp>0
                VServAdd=(ServPerRack/ServerPts(Rack))*((P_required(Rack)+dptemp)/C_D)^0.5-VServ_0(Rack)/ServerPts(Rack);
            else
                VServAdd=(ServPerRack/ServerPts(Rack))*(-(P_required(Rack)+dptemp)/C_D)^0.5-VServ_0(Rack)/ServerPts(Rack);
            end
        end
    else % Server fan speed assumed constant, regardless of local pressure drop
        if P_required(Rack)+dptemp>0
            VServAdd=(ServPerRack/ServerPts(Rack))*((P_required(Rack)+dptemp)/C_D)^0.5-VServ_0(Rack)/ServerPts(Rack); % Extra flow through the server at this cell due to local pressure
        else
            VServAdd=(ServPerRack/ServerPts(Rack))*(-(P_required(Rack)+dptemp)/C_D)^0.5-VServ_0(Rack)/ServerPts(Rack); % Extra flow through the server at this cell due to local pressure
        end
        EServAdd=0;
    end

    EServ(i,j)=(EServ_0(Rack)/ServerPts(Rack))+EServAdd; % Store amendments to rack flow rates and power consumptions as calculated earlier
    VServ(i,j)=(VServ_0(Rack)/ServerPts(Rack))+VServAdd;
    VServChange=abs(VServ(i,j)/VServPrev-1); % Change in VServ since last iteration of while loop
    VServPrev=VServ(i,j);
end